#include "alg/alphabet.hpp"

// some standard dna alphabets 
// Include nl (\012), and cr (\015) to make sequence parsing eol 
// convention independent.
const char *Alphabet::reduced_dna_cstr = "AaCcGgTtNn\012\015";
const char *Alphabet::reduced_rna_cstr = "AaCcGgUuNn\012\015";
const char *Alphabet::reduced_nucleic_cstr = "AaCcGgTtUuNn\012\015";
//! this is the general iupac alphabet for nucleotides
const char *Alphabet::nucleic_cstr =
  "AaCcGgTtUuRrYyMmKkSsWwBbDdHhVvNn-~.?\012\015";
//! the protein alphabet
const char *Alphabet::protein_cstr = 
  "AaCcDdEeFfGgHhIiKkLlMmNnPpQqRrSsTtVvWwYy\012\015";

const Alphabet& Alphabet::reduced_dna_alphabet() {
  static Alphabet *a = new Alphabet(reduced_dna_cstr);
  return *a;
}
const Alphabet& Alphabet::reduced_rna_alphabet() {
  static Alphabet *a = new Alphabet(reduced_rna_cstr);
  return *a;
}
const Alphabet& Alphabet::reduced_nucleic_alphabet() {
  static Alphabet *a = new Alphabet(reduced_nucleic_cstr);
  return *a;
}
const Alphabet& Alphabet::nucleic_alphabet() {
  static Alphabet *a = new Alphabet(nucleic_cstr);
  return *a;
}
const Alphabet& Alphabet::protein_alphabet() {
  static Alphabet *a = new Alphabet(protein_cstr);
  return *a;
}

Alphabet::Alphabet(const char *a) :
  alphabet(a)
{
  alphabet_set.insert(alphabet.begin(), alphabet.end());  
} 

void Alphabet::assign(const Alphabet& a)
{
  alphabet = a.alphabet;
  alphabet_set.clear();
  alphabet_set.insert(alphabet.begin(), alphabet.end());
}

bool Alphabet::exists(const char c) const
{
  return (alphabet_set.find(c) != alphabet_set.end());
}
