#define BOOST_AUTO_TEST_MAIN
#include <iostream>
#include <list>

#include <boost/test/auto_unit_test.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/path.hpp>
namespace fs = boost::filesystem;

#include "alg/flp.hpp"
#include "alg/sequence.hpp"
#include <iostream>
#include <list>

#include "mussa_exceptions.hpp"

BOOST_AUTO_TEST_CASE( flp_simple )
{
  FLPs f;
  f.setup(5, 4);
  // when we first setup progress should be equal to -1 
  // aka FLPs::seqcomp_not_running 
  BOOST_CHECK_EQUAL( f.progress(), -1 );
  f.seqcomp("AAGGTAAGGT", "AAGGTAAGGT", false);
  // now that we're done we should be equal to not running again.
  // yeah it'd be nice to make a thread test
  BOOST_CHECK_EQUAL( f.progress(), -1 );
  
  // are the match_locations correct?
  for (int i=0; i != f.size(); i++)
  {
    std::list<int> window_locations = f.match_locations(i);
    std::list<FLPs::match> window_matches   = f.matches(i);
    std::list<int>::iterator loc_i   = window_locations.begin();
    std::list<FLPs::match>::iterator match_i = window_matches.begin();
    for (; loc_i != window_locations.end(); ++loc_i, ++match_i)
    {
      // I'm missing pythons easy lists and in operator here.
      if (i == 0) BOOST_CHECK( *loc_i == 0 || *loc_i == 5);
      else if (i == 1) BOOST_CHECK( *loc_i == 1 || *loc_i == 6);
      else if (i == 2) BOOST_CHECK( *loc_i == 2 );
      else if (i == 3) BOOST_CHECK( *loc_i == 3 );
      else if (i == 4) BOOST_CHECK( *loc_i == 4 );
      else if (i == 6) BOOST_CHECK( *loc_i == 5 || *loc_i == 0);
      BOOST_CHECK_EQUAL( *loc_i, match_i->index );
    }
  }
}

BOOST_AUTO_TEST_CASE( flp_save )
{
  std::string seq("AAGGCCTTAAGGCCTT");
  int win_size = 4;
  FLPs f1;
  FLPs f2;
  f1.setup(win_size,3);
  f1.seqcomp(seq, seq, false);
  std::string fname("flp_save_this_is_a_horrible_filename_because_"
                    "im_too_lazy_to_write_a_portable_mktmp.flp");
                    
  f1.save(fname);
  f2.load(fname);
  boost::filesystem::remove( boost::filesystem::path(fname) );

  BOOST_CHECK_EQUAL( f1.size(), seq.size()-win_size+1);
  BOOST_CHECK_EQUAL( f2.size(), seq.size()-win_size+1);
  BOOST_CHECK_EQUAL( f1.size(), f2.size() );
  for (int win=0; win < f1.size(); ++win)
  {
    std::list<FLPs::match> f1_matches = f1.matches(win);
    std::list<FLPs::match> f2_matches = f2.matches(win);
    std::list<FLPs::match>::const_iterator f1_match_i = f1_matches.begin();
    std::list<FLPs::match>::const_iterator f2_match_i = f2_matches.begin();
    for( ; 
         f1_match_i != f1_matches.end() && f2_match_i != f2_matches.end();
         ++f1_match_i, ++f2_match_i)
    {
      BOOST_CHECK_EQUAL( *f1_match_i, *f2_match_i);
    }
  }
}

/*! Apparently when we run multiple seqcomps we want to 
 *  save all the FLPs generated
 */
BOOST_AUTO_TEST_CASE( flp_reverse_compliment )
{
  FLPs f1;
  Sequence s1("AAAATTTT");
  Sequence s2("AACAGGGG");
  f1.setup(4,3);
  f1.seqcomp(s1, s2, false);
  f1.seqcomp(s1, s2.rev_comp(), true);

  FLPs f2;
  f2.setup(4,3);
  f2.seqcomp(s1, s2.rev_comp(), true);
  f2.seqcomp(s1, s2, false);

  // The order of doing the reverse compliment search shouldn't matter
  // when we're using exactly the same sequence
  BOOST_CHECK_EQUAL( f1.size(), f2.size() );
  for (int i = 0; i < f1.size(); ++i )
  {
    BOOST_CHECK_EQUAL( f1.matches(i).size(), f2.matches(i).size() );
    // FIXME: should we test the individual lists?
  }
}

BOOST_AUTO_TEST_CASE( flp_file_check )
{
  fs::path filename = "Thy_micturations_are_to_me_as_plurdled_gabbleblotchits";
  FLPs f1;
  BOOST_CHECK_THROW( f1.load(filename), mussa_load_error);
}
