#ifndef MOTIFELEMENT_HPP_
#define MOTIFELEMENT_HPP_

#include <QColor>
#include <QString>

#include "alg/color.hpp"
#include "alg/sequence.hpp"

class MotifElement {
public:
  MotifElement();
  MotifElement(const Sequence& seq, Color c=Color(1.0,0.0,0.0,1.0));

  //! is the element "empty"
  bool isEmpty() const;
  
  //! should we search the analysis for this element?
  bool isEnabled() const;
  //! set the state for searching the analysis for this element?
  void setEnabled(bool);
  
  //! return color should this motif be drawn as
  Color getColor() const;
  //! set what color this motif should be drawn as
  void setColor(const Color& c);
  //! return QColor should this motif should be drawn as
  QColor getQColor() const;
  //! set color using QColor
  void setQColor(const QColor& c);
  
  //! return sequence
  const Sequence& getSequence() const;
  //! set sequence
  void setSequence(const Sequence& seq);
  //! set sequence text
  void setSequence(const std::string& seq_text);
  //! get sequence text
  QString getSequenceText() const;
  
  //! get sequence name
  std::string getName() const;
  //! set sequence name
  void setName(const std::string& seq_name);
  
protected:
  //! search for motif
  bool enabled;
  //! store motif color
  Color color;
  //! store sequence
  Sequence motif;
};
#endif /*MOTIFELEMENT_HPP_*/
